"""NextDNS constants."""
API_ENDPOINT = "https://api.nextdns.io"

API_CACHE_BOOST = "cacheBoost"
API_CNAME_FLATTENING = "cnameFlattening"
API_ECS = "ecs"
API_ALLOW_AFFILIATE = "allowAffiliate"
API_DISGUISED_TRACKERS = "disguisedTrackers"
API_AI_THREAT_TETECTION = "aiThreatDetection"
API_CSAM = "csam"
API_DDNS = "ddns"
API_NRD = "nrd"
API_PARKING = "parking"
API_CRYPTOJACKING = "cryptojacking"
API_DGA = "dga"
API_DNS_REBINDING = "dnsRebinding"
API_GOOGLE_SAFE_BROWSING = "googleSafeBrowsing"
API_IDN_HOMOGRAPHS = "idnHomographs"
API_THREAT_INTELLIGENCE_FEEDS = "threatIntelligenceFeeds"
API_TYPOSQUATTING = "typosquatting"
API_BLOCK_BYPASS = "blockBypass"
API_SAFESEARCH = "safeSearch"
API_YOUTUBE_RESTRICTED_MODE = "youtubeRestrictedMode"

ATTR_ANALYTICS = "analytics"
ATTR_ENABLED = "enabled"
ATTR_NAME = "name"
ATTR_PARENTAL_CONTROL = "parental_control"
ATTR_PERFORMANCE = "performance"
ATTR_PRIVACY = "privacy"
ATTR_PROFILE = "profile"
ATTR_PROFILES = "profiles"
ATTR_SECURITY = "security"
ATTR_SECURITY = "security"
ATTR_SETTINGS = "settings"
ATTR_TEST = "test"
ATTR_URL = "url"
ATTR_CLEAR_LOGS = "clear_logs"

ATTR_BLOCK_PAGE = "block_page"
ATTR_CACHE_BOOST = "cache_boost"
ATTR_CNAME_FLATTENING = "cname_flattening"
ATTR_ANONYMIZED_ECS = "anonymized_ecs"
ATTR_LOGS = "logs"
ATTR_WEB3 = "web3"

ATTR_ALLOW_AFFILIATE = "allow_affiliate"
ATTR_BLOCK_DISGUISED_TRACKERS = "block_disguised_trackers"

ATTR_AI_THREAT_DETECTION = "ai_threat_detection"
ATTR_BLOCK_CSAM = "block_csam"
ATTR_BLOCK_DDNS = "block_ddns"
ATTR_BLOCK_NRD = "block_nrd"
ATTR_BLOCK_PARKED_DOMAINS = "block_parked_domains"
ATTR_CRYPTOJACKING_PROTECTION = "cryptojacking_protection"
ATTR_DGA_PROTECTION = "dga_protection"
ATTR_DNS_REBINDING_PROTECTION = "dns_rebinding_protection"
ATTR_GOOGLE_SAFE_BROWSING = "google_safe_browsing"
ATTR_IDN_HOMOGRAPH_ATTACKS_PROTECTION = "idn_homograph_attacks_protection"
ATTR_THREAT_INTELLIGENCE_FEEDS = "threat_intelligence_feeds"
ATTR_TYPOSQUATTING_PROTECTION = "typosquatting_protection"

ATTR_BLOCK_BYPASS_METHODS = "block_bypass_methods"
ATTR_SAFESEARCH = "safesearch"
ATTR_YOUTUBE_RESTRICTED_MODE = "youtube_restricted_mode"

ENDPOINTS = {
    ATTR_ANALYTICS: "https://api.nextdns.io/profiles/{profile_id}/analytics/{type}",
    ATTR_CLEAR_LOGS: "https://api.nextdns.io/profiles/{profile_id}/logs",
    ATTR_PROFILE: "https://api.nextdns.io/profiles/{profile_id}",
    ATTR_PROFILES: "https://api.nextdns.io/profiles",
    ATTR_TEST: "https://{profile_id}.test.nextdns.io",
    ATTR_SECURITY: "https://api.nextdns.io/profiles/{profile_id}/security",
    ATTR_SETTINGS: "https://api.nextdns.io/profiles/{profile_id}/settings",
    ATTR_PERFORMANCE: "https://api.nextdns.io/profiles/{profile_id}/settings/performance",
    ATTR_PRIVACY: "https://api.nextdns.io/profiles/{profile_id}/privacy",
    ATTR_SECURITY: "https://api.nextdns.io/profiles/{profile_id}/security",
    ATTR_PARENTAL_CONTROL: "https://api.nextdns.io/profiles/{profile_id}/parentalControl",
    ATTR_LOGS: "https://api.nextdns.io/profiles/{profile_id}/settings/logs",
    ATTR_BLOCK_PAGE: "https://api.nextdns.io/profiles/{profile_id}/settings/blockPage",
}

MAP_DNSSEC = {False: "not_validated_queries", True: "validated_queries"}
MAP_ENCRYPTED = {False: "unencrypted_queries", True: "encrypted_queries"}
MAP_IP_VERSIONS = {4: "ipv4_queries", 6: "ipv6_queries"}
MAP_PROFILE = {"parentalControl": "parental_control"}
MAP_PROTOCOLS = {
    "DNS-over-HTTPS": "doh_queries",
    "DNS-over-QUIC": "doq_queries",
    "DNS-over-TLS": "dot_queries",
    "TCP": "tcp_queries",
    "UDP": "udp_queries",
}
MAP_STATUS = {
    "allowed": "allowed_queries",
    "blocked": "blocked_queries",
    "default": "default_queries",
    "relayed": "relayed_queries",
}

MAP_SETTING = {  # pylint: disable=consider-using-namedtuple-or-dataclass
    ATTR_BLOCK_PAGE: {
        ATTR_URL: ENDPOINTS[ATTR_BLOCK_PAGE],
        ATTR_NAME: ATTR_ENABLED,
    },
    ATTR_CACHE_BOOST: {
        ATTR_URL: ENDPOINTS[ATTR_PERFORMANCE],
        ATTR_NAME: API_CACHE_BOOST,
    },
    ATTR_CNAME_FLATTENING: {
        ATTR_URL: ENDPOINTS[ATTR_PERFORMANCE],
        ATTR_NAME: API_CNAME_FLATTENING,
    },
    ATTR_ANONYMIZED_ECS: {
        ATTR_URL: ENDPOINTS[ATTR_PERFORMANCE],
        ATTR_NAME: API_ECS,
    },
    ATTR_WEB3: {
        ATTR_URL: ENDPOINTS[ATTR_SETTINGS],
        ATTR_NAME: ATTR_WEB3,
    },
    ATTR_LOGS: {
        ATTR_URL: ENDPOINTS[ATTR_LOGS],
        ATTR_NAME: ATTR_ENABLED,
    },
    ATTR_ALLOW_AFFILIATE: {
        ATTR_URL: ENDPOINTS[ATTR_PRIVACY],
        ATTR_NAME: API_ALLOW_AFFILIATE,
    },
    ATTR_BLOCK_DISGUISED_TRACKERS: {
        ATTR_URL: ENDPOINTS[ATTR_PRIVACY],
        ATTR_NAME: API_DISGUISED_TRACKERS,
    },
    ATTR_AI_THREAT_DETECTION: {
        ATTR_URL: ENDPOINTS[ATTR_SECURITY],
        ATTR_NAME: API_AI_THREAT_TETECTION,
    },
    ATTR_BLOCK_CSAM: {
        ATTR_URL: ENDPOINTS[ATTR_SECURITY],
        ATTR_NAME: API_CSAM,
    },
    ATTR_BLOCK_DDNS: {
        ATTR_URL: ENDPOINTS[ATTR_SECURITY],
        ATTR_NAME: API_DDNS,
    },
    ATTR_BLOCK_NRD: {
        ATTR_URL: ENDPOINTS[ATTR_SECURITY],
        ATTR_NAME: API_NRD,
    },
    ATTR_BLOCK_PARKED_DOMAINS: {
        ATTR_URL: ENDPOINTS[ATTR_SECURITY],
        ATTR_NAME: API_PARKING,
    },
    ATTR_CRYPTOJACKING_PROTECTION: {
        ATTR_URL: ENDPOINTS[ATTR_SECURITY],
        ATTR_NAME: API_CRYPTOJACKING,
    },
    ATTR_DGA_PROTECTION: {
        ATTR_URL: ENDPOINTS[ATTR_SECURITY],
        ATTR_NAME: API_DGA,
    },
    ATTR_DNS_REBINDING_PROTECTION: {
        ATTR_URL: ENDPOINTS[ATTR_SECURITY],
        ATTR_NAME: API_DNS_REBINDING,
    },
    ATTR_GOOGLE_SAFE_BROWSING: {
        ATTR_URL: ENDPOINTS[ATTR_SECURITY],
        ATTR_NAME: API_GOOGLE_SAFE_BROWSING,
    },
    ATTR_IDN_HOMOGRAPH_ATTACKS_PROTECTION: {
        ATTR_URL: ENDPOINTS[ATTR_SECURITY],
        ATTR_NAME: API_IDN_HOMOGRAPHS,
    },
    ATTR_THREAT_INTELLIGENCE_FEEDS: {
        ATTR_URL: ENDPOINTS[ATTR_SECURITY],
        ATTR_NAME: API_THREAT_INTELLIGENCE_FEEDS,
    },
    ATTR_TYPOSQUATTING_PROTECTION: {
        ATTR_URL: ENDPOINTS[ATTR_SECURITY],
        ATTR_NAME: API_TYPOSQUATTING,
    },
    ATTR_BLOCK_BYPASS_METHODS: {
        ATTR_URL: ENDPOINTS[ATTR_PARENTAL_CONTROL],
        ATTR_NAME: API_BLOCK_BYPASS,
    },
    ATTR_SAFESEARCH: {
        ATTR_URL: ENDPOINTS[ATTR_PARENTAL_CONTROL],
        ATTR_NAME: API_SAFESEARCH,
    },
    ATTR_YOUTUBE_RESTRICTED_MODE: {
        ATTR_URL: ENDPOINTS[ATTR_PARENTAL_CONTROL],
        ATTR_NAME: API_YOUTUBE_RESTRICTED_MODE,
    },
}



"""Python wrapper for NextDNS API."""
from __future__ import annotations

import asyncio
import json
import logging
from collections.abc import Iterable
from http import HTTPStatus
from typing import Any, cast

from aiohttp import ClientSession

from .const import (
    API_AI_THREAT_TETECTION,
    API_ALLOW_AFFILIATE,
    API_BLOCK_BYPASS,
    API_CACHE_BOOST,
    API_CNAME_FLATTENING,
    API_CRYPTOJACKING,
    API_CSAM,
    API_DDNS,
    API_DGA,
    API_DISGUISED_TRACKERS,
    API_DNS_REBINDING,
    API_ECS,
    API_GOOGLE_SAFE_BROWSING,
    API_IDN_HOMOGRAPHS,
    API_NRD,
    API_PARKING,
    API_SAFESEARCH,
    API_THREAT_INTELLIGENCE_FEEDS,
    API_TYPOSQUATTING,
    API_YOUTUBE_RESTRICTED_MODE,
    ATTR_ANALYTICS,
    ATTR_CLEAR_LOGS,
    ATTR_ENABLED,
    ATTR_LOGS,
    ATTR_NAME,
    ATTR_PERFORMANCE,
    ATTR_PROFILE,
    ATTR_PROFILES,
    ATTR_TEST,
    ATTR_URL,
    ATTR_WEB3,
    ENDPOINTS,
    MAP_DNSSEC,
    MAP_ENCRYPTED,
    MAP_IP_VERSIONS,
    MAP_PROFILE,
    MAP_PROTOCOLS,
    MAP_SETTING,
    MAP_STATUS,
)
from .exceptions import (
    ApiError,
    InvalidApiKeyError,
    ProfileIdNotFoundError,
    ProfileNameNotFoundError,
    SettingNotSupportedError,
)
from .model import (
    AllAnalytics,
    AnalyticsDnssec,
    AnalyticsEncryption,
    AnalyticsIpVersions,
    AnalyticsProtocols,
    AnalyticsStatus,
    ConnectionStatus,
    Profile,
    ProfileInfo,
    Settings,
)

_LOGGER = logging.getLogger(__name__)


class NextDns:
    """Main class of NextDNS API wrapper."""

    def __init__(self, session: ClientSession, api_key: str) -> None:
        """Initialize NextDNS API wrapper."""
        self._session = session
        self._headers = {"X-Api-Key": api_key, "Content-Type": "application/json"}
        self._api_key = api_key
        self._profiles: list[ProfileInfo]

    @classmethod
    async def create(cls, session: ClientSession, api_key: str) -> NextDns:
        """Create a new instance."""
        instance = cls(session, api_key)
        await instance.initialize()

        return instance

    async def initialize(self) -> None:
        """Initialize."""
        _LOGGER.debug("Initializing with API Key: %s...", self._api_key[:10])
        self._profiles = list(self._parse_profiles(await self.get_profiles()))

    async def get_profiles(self) -> list[dict[str, str]]:
        """Get all profiles."""
        url = ENDPOINTS[ATTR_PROFILES]

        return cast(list[dict[str, str]], await self._http_request("get", url))

    async def get_profile(self, profile_id: str) -> Profile:
        """Get profile."""
        url = ENDPOINTS[ATTR_PROFILE].format(profile_id=profile_id)
        resp = await self._http_request("get", url)

        return Profile(
            **{MAP_PROFILE.get(key, key): value for key, value in resp.items()}
        )

    async def get_settings(self, profile_id: str) -> Settings:
        """Get profile settings."""
        profile_data = await self.get_profile(profile_id)

        return Settings(
            block_page=profile_data.settings["blockPage"][ATTR_ENABLED],
            cache_boost=profile_data.settings[ATTR_PERFORMANCE][API_CACHE_BOOST],
            cname_flattening=profile_data.settings[ATTR_PERFORMANCE][
                API_CNAME_FLATTENING
            ],
            anonymized_ecs=profile_data.settings[ATTR_PERFORMANCE][API_ECS],
            logs=profile_data.settings[ATTR_LOGS][ATTR_ENABLED],
            web3=profile_data.settings[ATTR_WEB3],
            allow_affiliate=profile_data.privacy[API_ALLOW_AFFILIATE],
            block_disguised_trackers=profile_data.privacy[API_DISGUISED_TRACKERS],
            ai_threat_detection=profile_data.security[API_AI_THREAT_TETECTION],
            block_csam=profile_data.security[API_CSAM],
            block_ddns=profile_data.security[API_DDNS],
            block_nrd=profile_data.security[API_NRD],
            block_parked_domains=profile_data.security[API_PARKING],
            cryptojacking_protection=profile_data.security[API_CRYPTOJACKING],
            dga_protection=profile_data.security[API_DGA],
            dns_rebinding_protection=profile_data.security[API_DNS_REBINDING],
            google_safe_browsing=profile_data.security[API_GOOGLE_SAFE_BROWSING],
            idn_homograph_attacks_protection=profile_data.security[API_IDN_HOMOGRAPHS],
            threat_intelligence_feeds=profile_data.security[
                API_THREAT_INTELLIGENCE_FEEDS
            ],
            typosquatting_protection=profile_data.security[API_TYPOSQUATTING],
            block_bypass_methods=profile_data.parental_control[API_BLOCK_BYPASS],
            safesearch=profile_data.parental_control[API_SAFESEARCH],
            youtube_restricted_mode=profile_data.parental_control[
                API_YOUTUBE_RESTRICTED_MODE
            ],
        )

    async def get_analytics_status(self, profile_id: str) -> AnalyticsStatus:
        """Get profile analytics status."""
        url = ENDPOINTS[ATTR_ANALYTICS].format(profile_id=profile_id, type="status")
        resp = await self._http_request("get", url)

        return AnalyticsStatus(
            **{MAP_STATUS[item["status"]]: item["queries"] for item in resp}
        )

    async def get_analytics_dnssec(self, profile_id: str) -> AnalyticsDnssec:
        """Get profile analytics dnssec."""
        url = ENDPOINTS[ATTR_ANALYTICS].format(profile_id=profile_id, type="dnssec")
        resp = await self._http_request("get", url)

        return AnalyticsDnssec(
            **{MAP_DNSSEC[item["validated"]]: item["queries"] for item in resp}
        )

    async def get_analytics_encryption(self, profile_id: str) -> AnalyticsEncryption:
        """Get profile analytics encryption."""
        url = ENDPOINTS[ATTR_ANALYTICS].format(profile_id=profile_id, type="encryption")
        resp = await self._http_request("get", url)

        return AnalyticsEncryption(
            **{MAP_ENCRYPTED[item["encrypted"]]: item["queries"] for item in resp}
        )

    async def get_analytics_ip_versions(self, profile_id: str) -> AnalyticsIpVersions:
        """Get profile analytics IP versions."""
        url = ENDPOINTS[ATTR_ANALYTICS].format(profile_id=profile_id, type="ipVersions")
        resp = await self._http_request("get", url)

        return AnalyticsIpVersions(
            **{MAP_IP_VERSIONS[item["version"]]: item["queries"] for item in resp}
        )

    async def get_analytics_protocols(self, profile_id: str) -> AnalyticsProtocols:
        """Get profile analytics protocols."""
        url = ENDPOINTS[ATTR_ANALYTICS].format(profile_id=profile_id, type="protocols")
        resp = await self._http_request("get", url)

        return AnalyticsProtocols(
            **{MAP_PROTOCOLS[item["protocol"]]: item["queries"] for item in resp}
        )

    async def connection_status(self, profile_id: str) -> ConnectionStatus:
        """Return True if the device is using NextDNS."""
        url = ENDPOINTS[ATTR_TEST].format(profile_id=profile_id)
        resp = await self._http_request("get", url)

        used_profile_id = None
        if status := resp["status"] == "ok":
            for item in self.profiles:
                if item.fingerprint == resp.get("profile"):
                    used_profile_id = item.id

        return ConnectionStatus(status, used_profile_id)

    async def clear_logs(self, profile_id: str) -> bool:
        """Get profile analytics dnssec."""
        url = ENDPOINTS[ATTR_CLEAR_LOGS].format(profile_id=profile_id)
        result = await self._http_request("delete", url)

        return result.get("success", False) is True

    async def get_all_analytics(self, profile_id: str) -> AllAnalytics:
        """Get profile analytics."""
        resp = await asyncio.gather(
            self.get_analytics_dnssec(profile_id),
            self.get_analytics_encryption(profile_id),
            self.get_analytics_ip_versions(profile_id),
            self.get_analytics_protocols(profile_id),
            self.get_analytics_status(profile_id),
        )

        return AllAnalytics(*resp)

    async def set_setting(self, profile_id: str, setting: str, state: bool) -> bool:
        """Toggle settings."""
        if setting not in MAP_SETTING:
            raise SettingNotSupportedError

        url = MAP_SETTING[setting][ATTR_URL].format(profile_id=profile_id)
        resp = await self._http_request(
            "patch", url, data={MAP_SETTING[setting][ATTR_NAME]: state}
        )

        return resp.get("success", False) is True

    async def _http_request(
        self, method: str, url: str, data: dict[str, Any] | None = None
    ) -> Any:
        """Make an HTTP request."""
        _LOGGER.debug("Requesting %s, method: %s, data: %s", url, method, data)

        if data:
            resp = await self._session.request(
                method, url, headers=self._headers, data=json.dumps(data)
            )
        else:
            resp = await self._session.request(method, url, headers=self._headers)

        _LOGGER.debug("Response status: %s", resp.status)

        if resp.status == HTTPStatus.FORBIDDEN.value:
            raise InvalidApiKeyError
        if resp.status == HTTPStatus.NO_CONTENT.value and method in ("delete", "patch"):
            return {"success": True}
        if resp.status != HTTPStatus.OK.value:
            result = await resp.json()
            raise ApiError(f"{resp.status}, {result['errors'][0]['code']}")

        result = await resp.json()

        return result["data"] if "data" in result else result

    def get_profile_name(self, profile_id: str) -> str:
        """Get profile name."""
        for profile in self.profiles:
            if profile.id == profile_id:
                return profile.name

        raise ProfileIdNotFoundError

    def get_profile_id(self, profile_name: str) -> str:
        """Get profile ID."""
        for profile in self.profiles:
            if profile.name == profile_name:
                return profile.id

        raise ProfileNameNotFoundError

    @staticmethod
    def _parse_profiles(profiles: list[dict[str, str]]) -> Iterable[ProfileInfo]:
        """Parse profiles."""
        for profile in profiles:
            yield ProfileInfo(profile["id"], profile["fingerprint"], profile["name"])

    @property
    def profiles(self) -> list[ProfileInfo]:
        """Return profiles."""
        return self._profiles



"""Type definitions for NextDNS."""
from __future__ import annotations

from dataclasses import dataclass
from typing import Any


@dataclass
class NextDnsData:
    """NextDNS data class."""


@dataclass
class AnalyticsStatus(NextDnsData):
    """AnalyticsStatus class."""

    all_queries: int = 0
    allowed_queries: int = 0
    blocked_queries: int = 0
    default_queries: int = 0
    relayed_queries: int = 0
    blocked_queries_ratio: float = 0

    def __post_init__(self) -> None:
        """Call after initialization."""
        self.all_queries = sum(
            [
                self.default_queries,
                self.blocked_queries,
                self.allowed_queries,
                self.relayed_queries,
            ]
        )

        self.blocked_queries_ratio = (
            0
            if not self.all_queries
            else round(self.blocked_queries / self.all_queries * 100, 1)
        )


@dataclass
class AnalyticsDnssec(NextDnsData):
    """AnalyticsDnssec class."""

    not_validated_queries: int = 0
    validated_queries: int = 0
    validated_queries_ratio: float = 0

    def __post_init__(self) -> None:
        """Call after initialization."""
        all_queries = sum([self.validated_queries, self.not_validated_queries])

        self.validated_queries_ratio = (
            0
            if not all_queries
            else round(self.validated_queries / all_queries * 100, 1)
        )


@dataclass
class AnalyticsEncryption(NextDnsData):
    """AnalyticsEncryption class."""

    encrypted_queries: int = 0
    unencrypted_queries: int = 0
    encrypted_queries_ratio: float = 0

    def __post_init__(self) -> None:
        """Call after initialization."""
        all_queries = sum([self.encrypted_queries, self.unencrypted_queries])

        self.encrypted_queries_ratio = (
            0
            if not all_queries
            else round(self.encrypted_queries / all_queries * 100, 1)
        )


@dataclass
class AnalyticsIpVersions(NextDnsData):
    """AnalyticsIpVersions class."""

    ipv6_queries: int = 0
    ipv4_queries: int = 0
    ipv6_queries_ratio: float = 0

    def __post_init__(self) -> None:
        """Call after initialization."""
        all_queries = sum([self.ipv6_queries, self.ipv4_queries])

        self.ipv6_queries_ratio = (
            0 if not all_queries else round(self.ipv6_queries / all_queries * 100, 1)
        )


@dataclass
class AnalyticsProtocols(NextDnsData):
    """AnalyticsProtocols class."""

    doh_queries: int = 0
    doq_queries: int = 0
    dot_queries: int = 0
    tcp_queries: int = 0
    udp_queries: int = 0
    doh_queries_ratio: float = 0
    doq_queries_ratio: float = 0
    dot_queries_ratio: float = 0
    tcp_queries_ratio: float = 0
    udp_queries_ratio: float = 0

    def __post_init__(self) -> None:
        """Call after initialization."""
        all_queries = sum(
            [
                self.doh_queries,
                self.doq_queries,
                self.dot_queries,
                self.tcp_queries,
                self.udp_queries,
            ]
        )

        self.doh_queries_ratio = (
            0 if not all_queries else round(self.doh_queries / all_queries * 100, 1)
        )
        self.doq_queries_ratio = (
            0 if not all_queries else round(self.doq_queries / all_queries * 100, 1)
        )
        self.dot_queries_ratio = (
            0 if not all_queries else round(self.dot_queries / all_queries * 100, 1)
        )
        self.tcp_queries_ratio = (
            0 if not all_queries else round(self.tcp_queries / all_queries * 100, 1)
        )
        self.udp_queries_ratio = (
            0 if not all_queries else round(self.udp_queries / all_queries * 100, 1)
        )


@dataclass
class AllAnalytics(NextDnsData):
    """AllAnalytics class."""

    dnssec: AnalyticsDnssec
    encryption: AnalyticsEncryption
    ip_versions: AnalyticsIpVersions
    protocols: AnalyticsProtocols
    status: AnalyticsStatus


@dataclass
class Profile(NextDnsData):
    """Profile class."""

    allowlist: list[dict[str, Any]]
    denylist: list[dict[str, Any]]
    fingerprint: str
    id: str
    name: str
    parental_control: dict[str, Any]
    privacy: dict[str, Any]
    rewrites: list
    security: dict[str, Any]
    settings: dict[str, Any]
    setup: dict[str, Any]


@dataclass
class Settings(NextDnsData):
    """Settings class."""

    block_page: bool
    cache_boost: bool
    cname_flattening: bool
    anonymized_ecs: bool
    logs: bool
    web3: bool

    allow_affiliate: bool
    block_disguised_trackers: bool

    ai_threat_detection: bool
    block_csam: bool
    block_ddns: bool
    block_nrd: bool
    block_parked_domains: bool
    cryptojacking_protection: bool
    dga_protection: bool
    dns_rebinding_protection: bool
    google_safe_browsing: bool
    idn_homograph_attacks_protection: bool
    threat_intelligence_feeds: bool
    typosquatting_protection: bool

    block_bypass_methods: bool
    safesearch: bool
    youtube_restricted_mode: bool


@dataclass
class ProfileInfo(NextDnsData):
    """ProfileInfo class."""

    id: str
    fingerprint: str
    name: str


@dataclass
class ConnectionStatus(NextDnsData):
    """ConnectionStatus class."""

    connected: bool
    profile_id: str | None = None



"""Setup module for nextdns."""
from pathlib import Path

from setuptools import setup

PROJECT_DIR = Path(__file__).parent.resolve()
README_FILE = PROJECT_DIR / "README.md"
VERSION = "1.0.1"


setup(
    name="nextdns",
    version=VERSION,
    author="Maciej Bieniek",
    description="Python wrapper for NextDNS API.",
    long_description=README_FILE.read_text(encoding="utf-8"),
    long_description_content_type="text/markdown",
    include_package_data=True,
    url="https://github.com/bieniu/nextdns",
    license="Apache-2.0 License",
    packages=["nextdns"],
    package_data={"nextdns": ["py.typed"]},
    python_requires=">=3.9",
    install_requires=list(
        val.strip() for val in open("requirements.txt", encoding="utf-8")
    ),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3 :: Only",
        "Typing :: Typed",
    ],
)



"""NextDNS exceptions."""


class NextDnsError(Exception):
    """Base class for nextdns errors."""


class InvalidApiKeyError(NextDnsError):
    """Raised to indicate invalid API key error."""


class ApiError(NextDnsError):
    """Raised to indicate API error."""

    def __init__(self, status: str) -> None:
        """Initialize."""
        super().__init__(status)
        self.status = status


class ProfileIdNotFoundError(NextDnsError):
    """Raised to indicate profile ID not found error."""


class ProfileNameNotFoundError(NextDnsError):
    """Raised to indicate profile name not found error."""


class SettingNotSupportedError(NextDnsError):
    """Raised to indicate setting not supported error."""
